﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Dades;
using System.Windows.Forms;

namespace Negoci
{
    // Classe que agrupa la lògica de l'aplicació relativa a les operacions amb reserves.
    public class Reserva
    {
        // Establim la connexió amb les classes que modelitzen la BD.
        DataClassesDataContext db = new DataClassesDataContext();

        // Mètode que permet recuperar una reserva de la BD pel seu identificador.
        public Reserves GetReserva(int id)
        {
            try
            {
                var query = (from c in db.Reserves
                             where c.idReserva == id
                             select c).FirstOrDefault();
                return query;
            }
            catch (Exception ex) { MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); return null; }            
        }
        
        // Mètode que retorna el llistat de les reserves enregistrades al sistema.
        public List<Reserves> LlistaReserves()
        {
            try
            {
                var query = (from q in db.Reserves
                            orderby q.data, q.hora ascending
                            select q).ToList();
                 return query;               
            }
            catch (Exception ex) { MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); return null; }            
        }

        // Mètode que permet enregistrar una reserva al sistema.
        public Boolean AfegirReserva(DateTime data, TimeSpan hora, int comensals, string comentaris, string nom, string cognom, string telefon, string correu)
        {
            try
            {
                // Mirem si hi ha places disponibles. Si no, no es fa la reserva.
                if (!this.ConsultarDisponibilitat(data, hora, comensals)) { return false; }

                // Si hi ha places disponibles, guardem la reserva.
                else
                {
                    Reserves r = new Reserves();

                    r.data = data;

                    r.hora = hora;

                    r.numComensals = comensals;

                    r.comentaris = comentaris;

                    // Afegim als comentaris les dades de la reserva (que no és d'un client registrat).
                    string info = ("\nNom:     " + nom + "\nCognom:  " + cognom + "\nTelefon: " + telefon + "\nCorreu:  " + correu);

                    r.comentaris += info;

                    db.Reserves.InsertOnSubmit(r);

                    db.SubmitChanges();

                    db.Connection.Close();

                    return true;
                }
            }
            catch (Exception ex) { MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); return false; }            
        }

        // Mètode que permet esborrar una reserva segons el seu identificador.
        public void EsborrarReserva(int id)
        {
            try
            {
                var query = (from q in db.Reserves
                             where q.idReserva == id
                             select q).FirstOrDefault();

                db.Reserves.DeleteOnSubmit(query);

                db.SubmitChanges();

                db.Connection.Close();
            }
            catch (Exception ex) { MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); }            
        }

        // Mètode que permet saber si hi ha places disponibles al restaurant un dia, una hora i per un nombre de comensals.
        // Per fer-ho, suposem que el restaurant té 12 taules de 4 comensals cadascuna.
        public Boolean ConsultarDisponibilitat(DateTime data, TimeSpan hora, int comensals)
        {
            try
            {
                int taulesOcupades = 0;

                Boolean resultat = false;

                // Mirem a la BD les reserves enregistrades per aquest dia i per l'hora (sencera).
                var query = (from q in db.Reserves
                             where q.data.Date.Equals(data.Date) && q.hora.Hours.Equals(hora.Hours)
                             select q);

                // Recorrem el resultat de la consulta i anem calculant quantes taules hi haurà ocupades tenint en compte
                // que cada reserva ocupa, com a mínim, una taula, encara que no l'ompli.
                if (query.Count() > 0)
                    foreach (Reserves q in query)
                    {
                        taulesOcupades += ((q.numComensals + 3) / 4);
                    }

                // En funció de les taules que ja hi ha ocupades i de les que necessitem per a la nova reserva, decidim.           
                taulesOcupades += ((comensals + 3) / 4);

                // Si s'ocupen menys taules de les disponibles (12) hi ha disponibilitat.
                if (taulesOcupades.CompareTo(13) < 0) { resultat = true; }

                // Si no, no hi ha disponibilitat.
                else { resultat = false; }

                return resultat;
            }
            catch (Exception ex) { MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); return false; }         
        }

        // Mètode que permet actualitzar els comentaris associats a una reserva donada.
        public void ActualitzarReserva(int id, string comentaris)
        {
            try
            {
                Dades.Reserves r = GetReserva(id);
                r.comentaris = comentaris;
                db.SubmitChanges();
                db.Connection.Close();
            }
            catch (Exception ex) { MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); }
        }
    }
}
